import re
from collections import OrderedDict

from resources.lib.api.api import API
from resources.lib.const import GENERAL, SETTINGS, STRINGS, lang_code_gui, COLOR, ROUTE
from resources.lib.routing.router import router
from resources.lib.storage.settings import settings
from resources.lib.utils.kodiutils import strip_accents, translate_genres, colorize


class MediaInfoRenderer:
    class TITLE:
        @staticmethod
        def default(media, *args):
            info_labels = media.get('info_labels')
            title = info_labels.get('title')
            return MediaInfoRenderer.build_title(media, title.encode('utf-8'), info_labels)

        @staticmethod
        def mixed(media, *args):
            info_labels = media.get('info_labels')
            titles = info_labels.get('parent_titles', []) or []
            titles.append(info_labels.get('title'))
            title = ' - '.join(titles)
            return MediaInfoRenderer.build_title(media, title.encode('utf-8'), info_labels)

        @staticmethod
        def a_z(media, letters):
            info_labels = media.get('info_labels')
            languages, labels_dict = MediaInfoRenderer.get_language_priority_and_lang_map(media, True)
            title = MediaInfoRenderer.get_az_title(letters, languages, labels_dict) or info_labels.get('originaltitle')
            return MediaInfoRenderer.build_title(media, title.encode('utf-8'), info_labels)

        @staticmethod
        def download_queue(media, *args):
            info_labels = media.get('info_labels')
            percentage = media.get('download_perc')
            speed = media.get('download_speed')
            title = STRINGS.DOWNLOAD_TITLE.format(progress=str(percentage),
                                                  title=info_labels.get('title').encode('utf-8'))
            return MediaInfoRenderer.build_title(media, title, info_labels)

        @staticmethod
        def _get_root_title(info_labels):
            return info_labels.get('parent_titles')[0].encode('utf-8')

        @staticmethod
        def numbering_builder(info_labels):
            season = info_labels.get('season')
            episode = info_labels.get('episode')
            if season and episode:
                numbering = STRINGS.SEASON_EPISODE_TITLE.format(season=str(season).zfill(2),
                                                                episode=str(episode).zfill(2))
            elif season and not episode:
                numbering = STRINGS.SEASON_TITLE.format(season=str(season).zfill(2))
            else:
                numbering = STRINGS.EPISODE_TITLE.format(episode=str(episode).zfill(2))
            return numbering

        @staticmethod
        def subtitles_search(info_labels):
            season = info_labels.get('season')
            episode = info_labels.get('episode')
            if season or episode:
                numbering = MediaInfoRenderer.TITLE.numbering_builder(info_labels)
                title = MediaInfoRenderer.TITLE._get_root_title(info_labels.get('labels_eng')) or \
                        MediaInfoRenderer.TITLE._get_root_title(info_labels)
                title = STRINGS.SUBTITLES_SEARCH_TV_SHOW_TITLE.format(title=title, numbering=numbering)
                info_labels.update({'tvshowtitle': title})
            else:
                title = info_labels.get('labels_eng', {}).get('title', info_labels.get('originaltitle',
                                                                                       info_labels.get('sorttitle',
                                                                                                       'N/A'))).encode(
                    'utf-8')
                title = STRINGS.SUBTITLES_SEARCH_TITLE.format(title=title, year=str(info_labels.get('year')))

            return title

    @staticmethod
    def build_title(media, title, info_labels):
        title = title
        info_labels.update({'sorttitle': title})
        season = info_labels.get('season')
        episode = info_labels.get('episode')
        audio = MediaInfoRenderer.preferred_lang_audio(media)

        if audio:
            title = STRINGS.TITLE_AUDIO_INFO.format(audio=lang_code_gui.get(audio), title=title)

        if season or episode:
            numbering = MediaInfoRenderer.TITLE.numbering_builder(info_labels)
            title = STRINGS.TV_SHOW_TITLE.format(title=title, numbering=numbering)
        else:
            title = STRINGS.TITLE_GENRE_YEAR.format(
                title=title,
                genre=MediaInfoRenderer.process_genres(info_labels.get('genre')),
                year=info_labels.get('year')
            )

        if not MediaInfoRenderer.stream_available(media) and media.get('children_count', 0) == 0:
            title = STRINGS.TITLE_STREAM_AVAILABLE.format(title=title)

        return title

    @staticmethod
    def merge_info_labels(media):
        translated_labels, art = MediaInfoRenderer.get_translated_labels(media)
        info_labels = media.get('info_labels')
        info_labels.update(translated_labels)
        return info_labels, art

    @staticmethod
    def stream_available(media):
        stream_count = media.get('available_streams', {}).get('count', 0)
        return stream_count > 0

    @staticmethod
    def preferred_lang_audio(media):
        langs = media.get('available_streams', {}).get('audio_languages')
        langs_settings = settings.get_languages(SETTINGS.PREFERRED_LANGUAGE, SETTINGS.FALLBACK_LANGUAGE)
        if langs:
            for lang_s in langs_settings:
                if lang_s and any(x.get('lang') == lang_s for x in langs):
                    return lang_s
        return ''

    @staticmethod
    def process_genres(genres):
        return ' / '.join(translate_genres(genres[:settings.as_int(SETTINGS.GENRES_COUNT)])).encode('utf-8')

    @staticmethod
    def get_az_title(letters, languages, labels_dict, when_missing=None):
        for lang_name in languages:
            value = labels_dict[lang_name].get('title')
            if value:
                stripped_value = re.sub(r'[^A-Za-z0-9]+|\s+', '', strip_accents(value)).lower()
                if stripped_value.startswith(letters.lower()):
                    return value
        return when_missing

    @staticmethod
    def get_localized_item(languages, labels_dict, key, when_missing=None):
        for lang_name in languages:
            value = labels_dict[lang_name].get(key)
            if value:
                if isinstance(value, list):
                    for index, item in enumerate(value):
                        if not item:
                            for lang_name2 in languages:
                                value2 = labels_dict[lang_name2].get(key)[index]
                                if value2:
                                    value[index] = value2
                                    break
                return value

        return when_missing

    @staticmethod
    def get_localized_item_nested(languages, labels_dict, dict_key, key, when_missing='invalid-url'):
        for lang_name in languages:
            value = labels_dict[lang_name].get(dict_key, {}).get(key)
            if value:
                return value
        return when_missing

    @staticmethod
    def get_language_priority_and_lang_map(media, all_languages=False):
        labels = media.get('i18n_info_labels')
        original_lang = media.get('language')
        labels_dict = {}

        preferred = settings.get_select(SETTINGS.PREFERRED_LANGUAGE)
        fallback = settings.get_select(SETTINGS.FALLBACK_LANGUAGE)
        langs = [preferred]
        if fallback:
            langs.append(fallback)
        for item in labels:
            lang = item.get('lang')
            labels_dict[lang] = item
            if original_lang == lang:
                langs.append(original_lang)

        langs.append(GENERAL.DEFAULT_LANGUAGE)
        if all_languages:
            available_langs_other = set([v['lang'] for v in labels]) - {preferred, fallback}
            langs = langs + list(available_langs_other)

        return list(OrderedDict.fromkeys(langs)), labels_dict

    @staticmethod
    def get_translated_labels_all(media, new_dict):
        languages, labels_dict = MediaInfoRenderer.get_language_priority_and_lang_map(media, True)
        original_title = media.get('info_labels').get('originaltitle')
        if new_dict['title'] is None and original_title is None:
            new_dict['title'] = MediaInfoRenderer.get_localized_item(languages, labels_dict, 'title')
        elif new_dict['title'] is None and original_title:
            new_dict['title'] = original_title
        new_dict['labels_eng'] = labels_dict.get('en')
        if new_dict['parent_titles']:
            for item in new_dict['parent_titles']:
                if not item:
                    new_dict['parent_titles'] = MediaInfoRenderer.get_localized_item(languages, labels_dict,
                                                                                     'parent_titles')
                    break
        return new_dict

    @staticmethod
    def get_translated_labels(media):
        languages, labels_dict = MediaInfoRenderer.get_language_priority_and_lang_map(media)
        new_dict = {}
        art = {}
        for label_name in ['title', 'plot', 'plotoutline', 'rating', 'votes', 'tagline', 'trailer', 'parent_titles']:
            new_dict[label_name] = MediaInfoRenderer.get_localized_item(languages, labels_dict, label_name)

        for label_name in ['poster', 'fanart', 'thumb', 'banner', 'clearart', 'clearlogo', 'landspace', 'icon']:
            art[label_name] = MediaInfoRenderer.get_localized_item_nested(languages, labels_dict, 'art', label_name)

        MediaInfoRenderer.get_translated_labels_all(media, new_dict)
        return new_dict, art

    @staticmethod
    def get_trailer_url(media):
        info_labels = media.get('info_labels')
        trailer = info_labels.get('trailer')
        return router.get_url(ROUTE.PLAY_TRAILER, video_url=trailer)
